/******************************************************************************
 * OpenLP - Open Source Lyrics Projection                                      *
 * --------------------------------------------------------------------------- *
 * Copyright (c) 2011-2014 Raoul Snyman                                        *
 * Portions copyright (c) 2011-2014 Tim Bentley, Johan Mynhardt, Samuel        *
 * Sjöbergsson                                                                 *
 * --------------------------------------------------------------------------- *
 * This program is free software; you can redistribute it and/or modify it     *
 * under the terms of the GNU General Public License as published by the Free  *
 * Software Foundation; version 2 of the License.                              *
 *                                                                             *
 * This program is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       *
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    *
 * more details.                                                               *
 *                                                                             *
 * You should have received a copy of the GNU General Public License along     *
 * with this program; if not, write to the Free Software Foundation, Inc., 59  *
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA                          *
 *******************************************************************************/
package org.openlp.android.common;

import java.io.*;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLDecoder;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.*;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import org.openlp.android.R;

/**
 * Personalised HttpClient to be used throughout OpenLP with customisable
 * parameters.
 */
public class OpenLPHttpClient extends DefaultHttpClient {

    private final String LOG_TAG = OpenLPHttpClient.class.getName();
    private final String urlBase;
    private HttpGet httpGet;
    private URL url;
    private SSLSocketFactory sslSocketFactory;

    public OpenLPHttpClient(Context context) {
        Map<String, ?> preferences = context.getSharedPreferences(context.getString(R.string.keySharedPreferences), Context.MODE_PRIVATE).getAll();

        HttpParams httpParams = new BasicHttpParams();
        httpGet = new HttpGet();

        Boolean useSsl = getPreference(preferences, context.getString(R.string.key_ssl_use), false);
        String host = getPreference(preferences, context.getString(R.string.keyHost),
                context.getString(R.string.hostDefaultValue));
        String port = getPreference(preferences, context.getString(R.string.keyPort), "4316");

        SchemeRegistry schemeRegistry = getConnectionManager().getSchemeRegistry();
        if (!useSsl) {
            schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), Integer.valueOf(port)));
        } else {
            try {
                initSsl();
                schemeRegistry.register(new Scheme("https", sslSocketFactory, Integer.valueOf(port)));
            } catch (NoSuchAlgorithmException e) {
                Log.e(LOG_TAG, e.toString());
            } catch (KeyManagementException e) {
                Log.e(LOG_TAG, e.toString());
            } catch (KeyStoreException e) {
                Log.e(LOG_TAG, e.toString());
            } catch (UnrecoverableKeyException e) {
                Log.e(LOG_TAG, e.toString());
            } catch (CertificateException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }

        String userid = getPreference(preferences, context.getString(R.string.key_userid), "openlp");

        String password = getPreference(preferences, context.getString(R.string.key_password), "password");

        Log.d(LOG_TAG, "Credentials set to " + userid + " : " + password);

        Credentials creds = new UsernamePasswordCredentials(userid, password);
        getCredentialsProvider().setCredentials(new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT), creds);

        urlBase = String.format("http%s://%s:%s", useSsl ? "s" : "", host, port);

        int connectionTimeout = context.getResources().getInteger(
                R.integer.connectionTimeoutDefaultValue);
        int socketTimeout = context.getResources().getInteger(
                R.integer.socketTimeoutDefaultValue);

        if (getPreference(preferences, context.getString(R.string.keyEnableCustomTimeout), false)) {
            Log.d(LOG_TAG, "Overriding Connection and Socket timeouts");

            connectionTimeout = getPreference(preferences,
                    context.getString(R.string.keyConnectionTimeout),
                    context.getResources().getInteger(
                            R.integer.connectionTimeoutDefaultValue)
            );

            socketTimeout = getPreference(preferences,
                    context.getString(R.string.keySocketTimeout),
                    context.getResources().getInteger(
                            R.integer.socketTimeoutDefaultValue)
            );
        }
        HttpConnectionParams.setConnectionTimeout(httpParams, connectionTimeout);
        HttpConnectionParams.setSoTimeout(httpParams, socketTimeout);
        setParams(httpParams);
    }

    private String getPreference(Map preferences, String key, String defaultValue) {
        if (preferences.containsKey(key)) {
            return preferences.get(key).toString();
        } else {
            return defaultValue;
        }
    }

    private Integer getPreference(Map preferences, String key, Integer defaultValue) {
        if (preferences.containsKey(key)) {
            return Integer.valueOf(preferences.get(key).toString());
        } else {
            return defaultValue;
        }
    }

    private Boolean getPreference(Map preferences, String key, Boolean defaultValue) {
        if (preferences.containsKey(key)) {
            return Boolean.valueOf(preferences.get(key).toString());
        } else {
            return defaultValue;
        }
    }

    public URL getUrl() {
        return url;
    }

    public String getUrlBase(){
        return urlBase;
    }

    public void setUrl(String apiPart) throws URISyntaxException, MalformedURLException {
        url = new URL(urlBase.concat(apiPart));
        try {
            Log.d(LOG_TAG, "URL set to: " + URLDecoder.decode(url.toString(), "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            Log.e(LOG_TAG, "Unable to decode url: " + e.getMessage());
        }
        httpGet.setURI(getUrl().toURI());
    }

    public String runUrl() {
        String response = "";
        try {
            HttpResponse execute = execute(httpGet);
            InputStream content = execute.getEntity().getContent();

            BufferedReader buffer = new BufferedReader(new InputStreamReader(content));
            String s;
            while ((s = buffer.readLine()) != null) {
                response += s;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return response;
    }

    private void initSsl() throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException,
            UnrecoverableKeyException, IOException, CertificateException {
        final SSLContext sslContext = SSLContext.getInstance("TLS");
        X509TrustManager trustManager = new X509TrustManager() {
            @Override
            public void checkClientTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {
            }

            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return null;
            }
        };
        sslContext.init(null, new TrustManager[]{trustManager}, null);

        KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
        keyStore.load(null, null);
        sslSocketFactory = new SSLSocketFactory(keyStore) {
            @Override
            public Socket createSocket() throws IOException {
                return sslContext.getSocketFactory().createSocket();
            }

            @Override
            public Socket createSocket(Socket socket, String host, int port, boolean autoClose) throws IOException {
                return sslContext.getSocketFactory().createSocket(socket, host, port, autoClose);
            }
        };
        sslSocketFactory.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
    }
}
